/*
 * Copyright 2014 OGIS-RI All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.co.ogis_ri.optaplanner.vehiclerouting.domain;

import jp.co.ogis_ri.optaplanner.vehiclerouting.domain.core.TimeWindowedStep;

/**
 * トラック
 * 
 * 
 */
public class Vehicle extends TimeWindowedStep {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * 積載可能量
	 */
	protected int capacity;

	/**
	 * 拠点
	 */
	protected Depot depot;

	/**
	 * ダミーフラグ
	 */
	protected boolean isDummy = false;

	/**
	 * 積載可能量を取得する
	 * 
	 * @return
	 */
	public int getCapacity() {
		return this.capacity;
	}

	/**
	 * 積載可能量を設定する
	 * 
	 * @param capacity
	 */
	public void setCapacity(final int capacity) {
		this.capacity = capacity;
	}

	/**
	 * 拠点を取得する
	 * 
	 * @return
	 */
	public Depot getDepot() {
		return this.depot;
	}

	/**
	 * 拠点を設定する
	 * 
	 * @param depot
	 */
	public void setDepot(final Depot depot) {
		this.depot = depot;
	}

	public boolean isDummy() {
		return this.isDummy;
	}

	public void setDummy(final boolean isDummy) {
		this.isDummy = isDummy;
	}

	// ************************************************************************
	// Complex methods
	// ************************************************************************

	/**
	 * 出発時間を取得する
	 */
	@Override
	public Integer getDepartureTime() {

		// 準備出来たら出発する
		return this.readyTime;
	}

	/**
	 * 
	 */
	@Override
	public Vehicle getVehicle() {
		return this;
	}

	/**
	 * 
	 */
	@Override
	public Location getLocation() {
		return this.depot.getLocation();
	}

	@Override
	public String toString() {
		return this.getLocation() + "[vehicle-" + this.id + "]";
	}

}
